/** ###################################################################
**     Filename  : Cpu.C
**     Processor : 56F84xx
**     Author    : Richy.Ye
**     Compiler  : Metrowerks DSP C Compiler
**     Date/Time : 03/05/2009
**     Abstract  :
**
**     Settings  :
**
**     Contents  :
**         EnableInt      - void Cpu_EnableInt(void);
**         DisableInt     - void Cpu_DisableInt(void);
**         GetResetSource - byte Cpu_GetResetSource(void);
**
**     (c) Freescale Semiconductor
**     2004 All Rights Reserved
**
**     (c) Copyright UNIS, a.s. 1997-2008
** ###################################################################*/

/* MODULE Cpu. */
#include "Types.h"
#include "common.h"
//#include "Events.h"
//#include "Cpu.h"
#include "Main.h"


SOC_CLOCKS soc_clk_freq;

#define STARTUP_CLOCK_INTERNAL_FAST_FREQ_TRIM_VALUE        ((*(uint16_t *)0xE42C) & 0x3FFU) /* Masked, not shifted 8MHz internal relaxation oscillator factory frequency trim value */

#define OCCS_OSCTL1_FREQ_TRIM8M_MASK  0x03FFU
#define OCCS_OSCTL1_FREQ_TRIM8M_BITNUM 0x00U
#define OCCS_OSCTL1_EXT_SEL_MASK      0x0400U
  #define OCCS_OSCTL1_OSC_DIV2_MASK     0x0800U
  #define OCCS_OSCTL1_CLK_MODE_MASK     0x1000U
  #define OCCS_OSCTL1_COHL_MASK         0x2000U
  #define OCCS_OSCTL1_ROSB_MASK         0x4000U
  #define OCCS_OSCTL1_ROPD_MASK         0x8000U
#define OCCS_CTRL_PRECS_MASK          0x0CU
  #define OCCS_CTRL_PRECS_BITNUM        0x02U
#define OCCS_STAT_LCK0_MASK           0x20U
  #define OCCS_CTRL_ZSRC_MASK           0x01U
  

#define STARTUP_OCCS_CTRL_VALUE                            0x81U     /* OCCS_CTRL value */ 
#define STARTUP_OCCS_DIVBY_VALUE                           0x2131U   /* OCCS_DIVBY value */ 
#define STARTUP_OCCS_STAT_VALUE                            0x00U     /* OCCS_STAT value */ 
#define STARTUP_OCCS_OSCTL1_VALUE                          0x0400 | STARTUP_CLOCK_INTERNAL_FAST_FREQ_TRIM_VALUE /* OCCS_OSCTL1 value */
#define STARTUP_OCCS_OSCTL2_VALUE                          0xC100UL  /* OCCS_OSCTL2 value */ 
#define STARTUP_OCCS_CLKCHKR_VALUE                         0x00U     /* OCCS_CLKCHKR value */ 
#define STARTUP_OCCS_CLKCHKT_VALUE                         0x00U     /* OCCS_CLKCHKT value */ 
#define STARTUP_OCCS_PROT_VALUE                            0x00U     /* OCCS_PROT value */ 

/* External clock pin routing */

/* No pins routed - external reference clock disabled. 
   See Clock settings\External reference clock property in CPU component */

/* SIM */

#define STARTUP_SIM_PCR_VALUE                              0x0400U   /* Peripheral clock rate settings constant */ 


#define OCCS_CTRL_PLLIE1_MASK                    0xC000u
#define OCCS_DIVBY_COD_MASK                      0xF00u
#define OCCS_CTRL_PLLIE0_MASK                    0x3000u
#define OCCS_CTRL_LOCIE_MASK                     0x800u
#define OCCS_DIVBY_PLLDB_MASK                    0x3Fu
#define OCCS_DIVBY_PWM_DIV2_MASK                 0x40u
#define PeriphSetBits16( Register, BitMask, BitValue )    (Register) = ((Register) & ~(BitMask)) | (BitValue)
#define PeriphClearBitMask16( Register, BitMask )  ((Register) &= ~(BitMask))
#define PeriphSetBitMask16( Register, BitMask )  ((Register) |= BitMask)
#define PeriphWriteReg16( Register, Data )  ( (Register) = Data )

/* Global variables */
extern UWord32 _vector_addr;
/*
** ===================================================================
**     Method      :  Cpu_DisableInt (bean 56F84xx)
**
**     Description :
**         Disables all maskable interrupts
**     Parameters  : None
**     Returns     : Nothing
** ===================================================================
*/
/*
void Cpu_DisableInt(void)

**      This method is implemented as macro in the header module. **
*/

/*
** ===================================================================
**     Method      :  Cpu_EnableInt (bean 56F8023)
**
**     Description :
**         Enables all maskable interrupts
**     Parameters  : None
**     Returns     : Nothing
** ===================================================================
*/
/*
void Cpu_EnableInt(void)

**      This method is implemented as macro in the header module. **
*/

/*
** ===================================================================
**     Method      :  Cpu_GetResetSource (bean 56F8023)
**
**     Description :
**         Gets low byte of the reset status register.
**     Parameters  : None
**     Returns     :
**         ---             - Content of the reset status register.
**                           You can use predefined constants listed
**                           bellow to find out a reset cause.
**                           Reset source (predefined constants):
**                           RSTSRC_POR - Power-on reset
**                           RSTSRC_PIN - External reset
**                           RSTSRC_LOR - COP Loss of Reference reset
**                           RSTSRC_TOR - COP Time-Out reset
**                           RSTSRC_SWR - Software reset
** ===================================================================
*/
/*
byte Cpu_GetResetSource(void)

**      This method is implemented as macro in the header module. **
*/

/*
** ===================================================================
**     Method      :  _EntryPoint (bean 56F8023)
**
**     Description :
**         Initializes the whole system like timing and so on. At the end 
**         of this function, the C startup is invoked to initialize stack,
**         memory areas and so on.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
extern void init_56800_(void);		/*Forward declaration of external startup function declared in startup file*/


void _EntryPoint(void)
{
  
  #pragma constarray off

  /*** ### 56F84xxx "Cpu" init code ... ***/
#ifdef FLASH
  setReg16(INTC_VBA, 0x0000);  /*Set the interrupt vectors base address*/
#else
  setReg16(INTC_VBA, 0x0600);  /*Set the interrupt vectors base address*/
#endif
  __DI();		/*Disable the maskable interrupts*/
  setReg16(COP_CTRL, 0x00);		/*Disable COP running after reset*/
 /*** PE initialization code after reset ***/
  /*System clock initialization*/
   PE_low_level_init();
   OCCS_Init();
    
  /*Set the flash clock prescaler*/
  //setReg16(HFM_CLKD, 0x28); 
  /*** End of PE initialization code after reset ***/
#ifndef CW_10_2
  asm(JMP init_56800_);		/*Jump to C startup code*/
#else
  //asm(JMP init_MC56F847xx_ISR_HW_RESET);		/*Jump to C startup code*/
#endif

}

/*
** ===================================================================
**     Method      :  PE_low_level_init (bean 56F8023)
**
**     Description :
**         Initializes beans and provides common register initialization. 
**         The method is called automatically as a part of the 
**         application initialization code.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
void PE_low_level_init(void)
{

  /*Configure SIM registers*/
 									/* Set up the peripheral clock rate register */
  setReg16(SIM_PCR, (SIM_PCR_PWM | SIM_PCR_SCI1_CR ) );	
  	  	  	  	  	  	  	  	  	/* PWM and SCI clock rate equals 2X bus clock rat,ie 100MHz
  	  	  	  	  	  	  	  	  	  	Set up the peripheral clock rate register */
  setReg16(SIM_PCE0, 0xE07F);		/* Enable clock to TA0, TA1, TA2, all GPIOs */
  setReg16(SIM_PCE1, 0x0000);		/* Set up the peripheral clock enable register 1 */
  setReg16(SIM_PCE2, 0x0000);		/* Set up the peripheral clock enable register 2 */
  setReg16(SIM_PCE3, 0x0000);		/* Set up the peripheral clock enable register 3 */
  setReg16(SIM_PROT, 0x0000);		/* Set up the SIM protection register */
  setReg16(SIM_CTRL, 0x0000);		/* Set up the SIM control register. dma enabled */
  setReg16(SIM_PWR, 0x0000);		/* Set up the SIM power control register */
  setReg16(SIM_CLKOUT, 0x0000);		/* Set up the SIM clock output select register */ 
  /* Shadow registers initialization */
  asm {
    moveu.w #0, R0
    moveu.w #0, R1
    moveu.w #0, N
    moveu.w #65535, M01
    nop
    nop
    swap shadows
  }

#ifdef PMC_TRIMMED
	 clrSetReg16Bits(PMC_CTRL, 0xF000, ((word)SIM_NVMOPT2L & 0xF000)); /* Set the trim osc freq with factory programmed value */
#else 
	 // trim PMC
	 clrSetReg16Bits(PMC_CTRL, 0xF000, (PMC_TRIM_VALUE<<12));
#endif
  
  
  clrReg16Bits(INTC_CTRL, 0x20);		/* Enable all interrupts */
  __EI(0);                              /* Enable interrupts of the selected priority level */
  
  
#ifndef DISABLE_BUS_INTERRUPT
  MCM_CFISR |=MCM_CFISR_CFEI; // bus error interrupt clear
  MCM_CFIER |=(MCM_CFIER_ECFEI);// bus error interrupt enabled.

  INTC_IPR0&= (~(INTC_IPR0_BUS_ERR_0));//clear bus error priority if any. change INTC_IPR1 to INTC_IPR0 for nevis2.
  INTC_IPR0|=INTC_IPR0_BUS_ERR_1 | INTC_IPR0_BUS_ERR_0;
  
#endif

}

/*
** ===================================================================
**     Method      :  OCCS_Init (void)
**
**     Description :
**         This function implement to initialize the on-chip clock 
**         synthesis circuit for proper operation.
** ===================================================================
*/
void OCCS_Init(void)
{
    /* System clock initialization */
  /*** End of PE initialization code after reset ***/

	PeriphSetBits16(OCCS_OSCTL1, OCCS_OSCTL1_FREQ_TRIM8M_MASK, STARTUP_OCCS_OSCTL1_VALUE & OCCS_OSCTL1_FREQ_TRIM8M_MASK); /* Frequency trim of the 8MHz internal clock source */ 
	PeriphClearBitMask16(OCCS_OSCTL1, 0x8000); /* Enable 8MHz internal relaxation oscillator */ 
	PeriphClearBitMask16(OCCS_CTRL, 0x0C); /* Select an internal 8MHz clock source for the CPU core */ 
	 
	/* OCCS_CTRL: LCKON=1,PLLPD=0 */
	PeriphSetBits16(OCCS_CTRL, 0x10, 0x80); /* Enable PLL and lock detector */ 
	PeriphSetBits16(OCCS_DIVBY, OCCS_DIVBY_COD_MASK | OCCS_DIVBY_PLLDB_MASK | OCCS_DIVBY_PWM_DIV2_MASK, STARTUP_OCCS_DIVBY_VALUE & (OCCS_DIVBY_COD_MASK | OCCS_DIVBY_PLLDB_MASK | OCCS_DIVBY_PWM_DIV2_MASK)); /* Set the clock prescaler register */ 
	while(!getRegBit(OCCS_STAT, LCK0)){} /* Wait for PLL lock */
	/* OCCS_CTRL: ZSRC=1 */
	PeriphSetBitMask16(OCCS_CTRL, 0x01); /* Select clock source from PLL */ 
	PeriphSetBits16(OCCS_CTRL, OCCS_CTRL_PLLIE1_MASK | OCCS_CTRL_PLLIE0_MASK | OCCS_CTRL_LOCIE_MASK, STARTUP_OCCS_CTRL_VALUE & (OCCS_CTRL_PLLIE1_MASK | OCCS_CTRL_PLLIE0_MASK | OCCS_CTRL_LOCIE_MASK)); /* Set PLL interrupts */ 
	
	PeriphWriteReg16(SIM_PCR, STARTUP_SIM_PCR_VALUE); /* Set the peripheral clock rate register */ 
	PeriphWriteReg16(OCCS_PROT, STARTUP_OCCS_PROT_VALUE); /* Set the peripheral clock protection register */ 
}
/* END Cpu. */

