/******************************************************************************
* 
* Copyright (c) 2012 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
/*!
********************************************************************************
*
* FILE NAME: SCI0_test.c   
*
* TARGET: MC56F8400 device
*
*******************************************************************************
*
* Detailed Description of the file.
*
***************************************************************************//*!

/*Include shared modules, which are used for whole project*/
#include "Types.h"
#include "derivative.h"
#include "SCI0_test.h"

/* FreeMASTER configuration header file */
#include "freemaster.h"

/*****************************************************************************
*
* Function: 		static void SCI0_Init(unsigned int uiBaudrate)
*
* Description: 		SCI0 module initialization for FreeMASTER Communication 
*
* Returns: 			None
*
* Global Data: 		None
*
* Arguments: 		uiBaudrate - in bauds, typically 9600
*
* Range Issues: 	None
*
* Special Issues: 	None
*
*****************************************************************************/

void SCI0_Init(unsigned int uiBaudrate)
{
	static uint16_t	ui16Sbr;
	static uint8_t	ui8Frac_sbr;

	/* Enable clock to SCI0 */
	SIM_PCE1 |= SIM_PCE1_SCI0;
					
	/* Configure GPIOF4 as TXD1 and GPIOF5 as RXD1 */
	SIM_PCE0  |= SIM_PCE0_GPIOC; /* Enable GPIOC Clock */
	GPIOC_PER |= GPIOC_PER_PE_2 | GPIOC_PER_PE_3; /* PER Enable for GPIOC2 and GPIOC3 */

	SIM_GPSCL = SIM_GPSCL & ~(SIM_GPSCL_C3 | SIM_GPSCL_C2);
	SIM_GPSCL |= (SIM_GPSCL_C3_1);
	
	/* baudrate calculation */
	ui16Sbr = (QSCI0_CLOCK_HZ>>4)/ uiBaudrate;
	ui8Frac_sbr = ((QSCI0_CLOCK_HZ>>1)/uiBaudrate - (ui16Sbr<<3)) & 0x7;
	QSCI0_RATE = (ui16Sbr<<3) | ui8Frac_sbr;

	/* enable transmitter and receiver */
	QSCI0_CTRL1 = (QSCI0_CTRL1_TE | QSCI0_CTRL1_RE);
	
}
/******************************************************************************/
/**************************         END          ******************************/
/******************************************************************************/
